library(coda)
library(MASS)
library(lattice)



source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")





read.log <- function(fpath) {
  x <- read.table(fpath, header=TRUE)
  xb <- x[burnin.for.MCMC:nrow(x),]
  mcmclog <- as.mcmc(xb)
  mcmclog
  }



est.migs <- function(sdir) {
	est.migcounts <- matrix(0, nrow=number.of.loci, ncol=number.of.replicates) 
	for (rep in first.replicate:last.replicate) {
		params.fpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/", "locus1.log")
		mcmc <- read.log(params.fpath)
		migcounts <- mcmc[, paste0("MigCount.gTreeWE.t.locus", 1:number.of.loci)]
    migcount.means <- colMeans(migcounts[,])
    est.migcounts[ , rep] <- migcount.means
	}
	est.migcounts
}







true.migs <- function(sdir) {
	true.migcounts <- matrix(0, nrow=number.of.loci, ncol=number.of.replicates) 
	
	if (is.timezero.allele(sdir)) {
		true.migcounts[1,] <- 1
	} else if (is.timezero.mig(sdir)) {
		true.migcounts[,] <- 1
	} else {
		for (rep in first.replicate:last.replicate) {
			true.migs.fpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/", "migs.txt")
			true.migs.lines <- readLines(con=true.migs.fpath)
			true.mig.tokens <- strsplit(true.migs.lines, split="[[:blank:]]+")
			for (loc in 1:length(true.migs.lines)) {
				n <- length(true.mig.tokens[[loc]])
				stopifnot(n >= 2)
				n <- n - 2
				stopifnot(n %% 5 == 0)
				n <- n/5
				true.migcounts[loc, rep] <- n
			}
		}
	}
	true.migcounts
}



yesno.means <- function(sdir) {
	est.mig.matrix <- est.migs(sdir)
	true.mig.matrix <- true.migs(sdir)
	ests0 <- double(0)
	estsSome <- double(0)
	for (rep in first.replicate:last.replicate) {
		for (loc in 1:number.of.loci) {
			tc <- true.mig.matrix[loc,rep] 
			if (tc == 0) {
				ests0[length(ests0) + 1] <- est.mig.matrix[loc,rep]
			} else {
				estsSome[length(estsSome) + 1] <- est.mig.matrix[loc,rep]
			}
		}
	}
	list(no=ests0, yes=estsSome)
}




draw.boxplot <- function(sdirs) {
	for (sdir in sdirs) {
		cat(sdir, "... ")
		xx <- yesno.means(sdir)
		cat("found yes/no migs\n")
		
		maxyes <- 0
		if (length(xx$yes) > 0) {
			maxyes <- max(xx$yes) * 1.02
		}
		if (maxyes < 2.0) {
			maxyes <- 2.0
		}
		
		
		boxplot(xx, 
						width=1+c(length(xx$no), length(xx$yes)), 
						xaxt="n", ylim=c(0,maxyes))
		axis(1, at=1:2, labels=c("0", ">=1"))
		title(main = label.for.plot(sdir))
	}
}



draw.boxplots.array <- function(layout.mat, sdirs) {
	layout(mat=layout.mat)
	par(mar=c(4,4,3,1)+.1)
	draw.boxplot(sdirs)
}
	


draw.boxplots.array.screen.and.pdf <- function(layout.mat, sdirs) {
	draw.boxplots.array(layout.mat, sdirs)
	pdf(file=fpath.for.mig.yes.no(), pointsize=9, width=6, height=8)
	draw.boxplots.array(layout.mat, sdirs)
	dev.off()
}


####################################################################################

do.migs.yes.no.spp.4 <- function() {
	layout.mat <- matrix(c(13,0,0,0, 1,2,3,4, 5,6,7,8, 9,10,11,12, 14,15,16,17), nrow=5, ncol=4, byrow=TRUE)
	draw.boxplots.array.screen.and.pdf(layout.mat, fourspp.all.scenario.dirs)
}







do.migs.yes.no.spp.10 <- function() {
	
	nnn <- 0
	if (just.coal.paraphyly) { nnn <- nnn + 1 }
	if (just.leache) { nnn <- nnn + 1 }
	if (just.coal.big.rates) { nnn <- nnn + 1 }
	stopifnot(nnn <= 1)
	
	
	if (just.coal.paraphyly) {
		layout.mat <- matrix(c(5,0,  1,2, 3,4, 6,7), nrow=4, ncol=2, byrow=TRUE)
		draw.boxplots.array.screen.and.pdf(layout.mat, tenspp.just.coal.paraphyly.dirs)
	} else if (just.leache) {
		stop("Not implemented in do.migs.yes.no.spp.10()")
	} else if (just.coal.big.rates) {
		stop("Not implemented in do.migs.yes.no.spp.10()")
	}	else {
		layout.mat <- matrix(c(19,0,0,0,0,0,  1,2,3,4,5,6,  7,8,9,10,11,12,  13,14,15,16,17,18,  20,21,22,23,24,25), nrow=5, ncol=6, byrow=TRUE)
		draw.boxplots.array.screen.and.pdf(layout.mat, tenspp.all.scenario.dirs)
	}
}


#################################################################################################


do.migs.yes.no <- function(nspp) {
	if (nspp == 4) {
		do.migs.yes.no.spp.4()
	}
	if (nspp == 10) {
		do.migs.yes.no.spp.10()
	}
}




