
# This uses MCcoal files supplied by Adam Leache.
# It copies them into my directory structure,
# then runs MCcoal (My-MCcoal) on them,
# then renames the sequence and tree files to my conventions,
# then makes the time-zero ones



##################################################################################
# File paths and names

this.directory <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Leache-TenSpp"
MyMCcoal.fpath <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/My-MCcoal.exe"


source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")




my.migpat.dpath <- function(migpat) {
	n.main <- nrow(ten.spp.mapping)
	if (migpat <= n.main) {
		paste0(BEASTresults.dir, "/", ten.spp.mapping[migpat, "my.dir"])
	} else {
		paste0(BEASTresults.dir, "/", tenspp.time.zero.dirs[migpat-n.main])
	}
}




my.migpat.rep.dpath <- function(migpat, rep) {
	my.repnum <- sprintf("%03d", rep)
	paste0(my.migpat.dpath(migpat), "/rep", my.repnum)
}



leache.ctrl.fpath <- function(migpat, rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(this.directory, "/ZCtrlFiles/", ten.spp.mapping[migpat, "leache.to.repnum"], leache.repnum, ".ctl")
}



leache.trees.leaf <- function(migpat, rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(ten.spp.mapping[migpat, "leaf.in.ctrl"], leache.repnum, ".tre")
}

leache.seq.leaf <- function(migpat, rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(ten.spp.mapping[migpat, "leaf.in.ctrl"], leache.repnum, ".txt")
}



#################################################################################
# copying from Leache's ctrl file names to mine


make.dir.if.needed <- function(dpath) {
  if (!dir.exists(dpath)) {
   	dir.create(dpath)
  }
}


make.my.migpat.dirs <- function() {
	for (s in 1:nof.tenspp.migpats) {
		make.dir.if.needed(my.migpat.dpath(s))
	}
}




copy.ctrl.files <- function(s) {
	all.ok <- TRUE
	for (rep in first.replicate:last.replicate) {
		coal.pat <- nrow(ten.spp.mapping)  # COAL is last one
		if (s <= nrow(ten.spp.mapping)) {
			from <- leache.ctrl.fpath(s, rep)
		} else {
			from <- leache.ctrl.fpath(coal.pat, rep)
		}
		to <- my.migpat.rep.dpath(s, rep)
		make.dir.if.needed(to)
		to <- paste0(to, "/", "ctrl.txt")
		ok <- file.copy(from, to, overwrite=TRUE)
		if (!ok) {
			cat("Failed to copy ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}
	}
	all.ok
}




#################################################################################
# Running MCCoal, and renamng output to my conventions


run.mccoal.one.scenario.one.rep <- function(scenario.rep.dir) {
  setwd(scenario.rep.dir)
  command <- paste0(MyMCcoal.fpath, " ctrl.txt")
  error.code <- system(command)
  if (error.code != 0) {
  	err.message <- paste("Error. Code=",  error.code,
  											 "running MCcoal with directory", scenario.rep.dir, "rep", rep)
    print(err.message)
  }
}




# Runs MCcoal on main migration pattern s (in 1:13) and renames the output
# sequence and tree files
run.mccoal.main.migs <- function(s) {
	all.ok <- TRUE
	for (rep in first.replicate:last.replicate) {
		run.mccoal.one.scenario.one.rep(my.migpat.rep.dpath(s, rep))
		dpath <- my.migpat.rep.dpath(s, rep)
		
		from <- paste0(dpath, "/", leache.seq.leaf(s, rep))
		to <- paste0(my.migpat.rep.dpath(s, rep), "/", "seq.txt")
		ok <- file.rename(from, to)
		if (!ok) {
			cat("Failed to rename\n ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}
		
		from <- paste0(dpath, "/", leache.trees.leaf(s, rep))
		to <- paste0(my.migpat.rep.dpath(s, rep), "/", "trees.txt")
		ok <- file.rename(from, to)
		if (!ok) {
			cat("Failed to rename\n ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}	
	}
	all.ok
}




######################################################################################################
# Making the 
# "D-E-1-allele", "D-E-1-mig", "F-E-1-allele", "F-E-1-mig", "F-G-1-allele", "F-G-1-mig
# sequence files from the No-mig ones



X4.to.Y5 <- function(lines, x, y, one) {
	y1i <- grep(paste0(y, "1^", toupper(y)), lines, fixed = TRUE)
	y2i <- grep(paste0(y, "2^", toupper(y)), lines, fixed = TRUE)
	y3i <- grep(paste0(y, "3^", toupper(y)), lines, fixed = TRUE) 
	y4i <- grep(paste0(y, "4^", toupper(y)), lines, fixed = TRUE)
	x4i <- grep(paste0(x, "4^", toupper(x)), lines, fixed = TRUE)
	if (one) {
		y1i <- y1i[1]
		y2i <- y2i[1]
		y3i <- y3i[1]
		y4i <- y4i[1]
		x4i <- x4i[1]
	}
	y1 <- lines[ y1i ]
	y2 <- lines[ y2i ]
	y3 <- lines[ y3i ]
	y4 <- lines[ y4i ]
	x4 <- lines[ x4i ]
	y5 <- gsub(paste0(x, "4^", toupper(x)), paste0(y, "5^", toupper(y)), x4, fixed = TRUE)
	lines[x4i] <- y1
	lines[y1i] <- y2
	lines[y2i] <- y3
	lines[y3i] <- y4
	lines[y4i] <- y5
	lines
}



one.D4.to.E5 <- function(lines) {
	X4.to.Y5(lines, "d", "e", TRUE)
}

one.F4.to.E5 <- function(lines) {
	X4.to.Y5(lines, "f", "e", TRUE)
}

one.F4.to.G5 <- function(lines) {
	X4.to.Y5(lines, "f", "g", TRUE)
}


all.D4.to.E5 <- function(lines) {
	X4.to.Y5(lines, "d", "e", FALSE)
}

all.F4.to.E5 <- function(lines) {
	X4.to.Y5(lines, "f", "e", FALSE)
}

all.F4.to.G5 <- function(lines) {
	X4.to.Y5(lines, "f", "g", FALSE)
}




sink.lines <- function(yy, fpath) {
	sink(fpath)
	cat("") # overwrites old file if any
	for (i in 1:length(lines)) {
		cat(yy[i], "\n", append = TRUE, sep="")
	}
	sink(NULL)
}



##########################################################################


make.my.migpat.dirs()


for (s in 1:nof.tenspp.migpats) {
	copy.ctrl.files(s)
}



# Run MCcoal on the ctrl files just copied, and rename the output
# sequence and tree files to my conventions
for (s in 1:nrow(ten.spp.mapping)) {
	ok <- run.mccoal.main.migs(s)
	if (!ok) {
		break
	}
}




# Make the time-zero sequences
for (rep in first.replicate:last.replicate) {
	coal.pat <- nrow(ten.spp.mapping)  # COAL is last one
	lines <- readLines(paste0(my.migpat.rep.dpath(coal.pat, rep), "/seq.txt"))
	
	pn <- 1
	stopifnot(tenspp.time.zero.dirs[pn] == "D-E-1-allele")
	rep.dir <- my.migpat.rep.dpath(coal.pat+pn, rep)
	make.dir.if.needed(rep.dir)
	sink.lines(one.D4.to.E5(lines), paste0(rep.dir, "/seq.txt"))
	
	pn <- 2
	stopifnot(tenspp.time.zero.dirs[pn] == "D-E-1-mig")
	rep.dir <- my.migpat.rep.dpath(coal.pat+pn, rep)
	make.dir.if.needed(rep.dir)
	sink.lines(all.D4.to.E5(lines), paste0(rep.dir, "/seq.txt"))
	
	pn <- 3
	stopifnot(tenspp.time.zero.dirs[pn] == "F-E-1-allele")
	rep.dir <- my.migpat.rep.dpath(coal.pat+pn, rep)
	make.dir.if.needed(rep.dir)
	sink.lines(one.F4.to.E5(lines), paste0(rep.dir, "/seq.txt"))
	
	pn <- 4
	stopifnot(tenspp.time.zero.dirs[pn] == "F-E-1-mig")
	rep.dir <- my.migpat.rep.dpath(coal.pat+pn, rep)
	make.dir.if.needed(rep.dir)
	sink.lines(all.F4.to.E5(lines), paste0(rep.dir, "/seq.txt"))
	
	pn <- 5
	stopifnot(tenspp.time.zero.dirs[pn] == "F-G-1-allele")
	rep.dir <- my.migpat.rep.dpath(coal.pat+pn, rep)
	make.dir.if.needed(rep.dir)
	sink.lines(one.F4.to.G5(lines), paste0(rep.dir, "/seq.txt"))
	
	pn <- 6
	stopifnot(tenspp.time.zero.dirs[pn] == "F-G-1-mig")
	rep.dir <- my.migpat.rep.dpath(coal.pat+pn, rep)
	make.dir.if.needed(rep.dir)
	sink.lines(all.F4.to.G5(lines), paste0(rep.dir, "/seq.txt"))
	
  cat("Done time zero sequence files for replicate", rep, "\n")
}



