library(coda)
library(MASS)

fpath <- "G:/DENIMtestsMay27-model10"





read.log <- function(fpath, burnin.fraction) {
  stopifnot(burnin.fraction <= 1)
  x <- read.table(fpath, header=TRUE)
  burnin <- as.integer(nrow(x)*burnin.fraction)
  xb <- x[burnin:nrow(x),]
  mcmclog <- as.mcmc(xb)
  mcmclog
}




lookat.mcmclog <- function(mcmclog, nspp, nindivs, g, ils, model) {
  effs <- effectiveSize(mcmclog)
  mig.count.names <- paste0("MigCount.gTreeWE.t.s", nspp, "i", nindivs, "locus", 1:g)
  main.effs <- effs[c("posterior", "prior", "smcCoalescent", "TreeHeight.Species", mig.count.names)]
  sH <- mean(mcmclog[,"TreeHeight.Species"])
  ESS <- main.effs["TreeHeight.Species"]
  true.SH <- sum(1 / (2:nspp)) / 100
  meanMC <- mean(mcmclog[,mig.count.names])
  ngtreebranchpairs <- nindivs*nspp - 1
  c(nspp, nindivs, g, ils, model, nindivs*nspp*g, sH, sH/true.SH, ESS, meanMC, meanMC/ngtreebranchpairs)
}







analysis.dir <- "G:/DENIM-prior-Aug5"

xml.fragments.dir <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Prior/Aug5"


ils.vals <- c("0.0005", "0.005", "0.05")
ils.txts <- c("0p0005", "0p005", "0p05")

priormean.vals <- c("0.00125", "0.005")
priormean.txts <- c("0p00125", "0p005")

relFact.vals <- c("1.0", "-1.0")
relFact.txts <- c("F", "S")



results <- matrix(nrow = 2*2*3*2*2, ncol = 11)
colnames(results) <- c("nspp", "nindivs", "nloci", "ils", "model", "nseqs", "sHeight", "ratio", "ESS", "mean.migs", "migs.pbp")
n <- 1
for (nindivs in c(3,9)) {
	for (g in c(3,9)) {
		for (ils in 1:3) {
			for (pm in 1:2) {
				for (rf in 1:2) {
					dpath <- paste0(analysis.dir, "/s6", "_i", nindivs, "_loci", g, "_ils", ils.txts[ils], 
													"_pm", priormean.txts[pm], "_", relFact.txts[rf])
					analysis.fpath <- paste0(dpath, "/", "s6i", nindivs, "locus1.log")
					mcmclog <- read.log(analysis.fpath, 0.5)
					results[n,] <- lookat.mcmclog(mcmclog, 6, nindivs, g, as.double(ils.vals[ils]), as.double(relFact.vals[rf]))
					cat(n, "\n")
					n <- n+1
				}
			}
		}
	}
}


print(results, digits=3)


xx <- 1:(2*2*3)
xxnames <- rep("", length(xx))
ratios <- matrix(0, ncol=4, nrow=length(xx))
migs <- matrix(0, ncol=4, nrow=length(xx))
n <- 1
for (nindivs in c(3,9)) {
	for (g in c(3,9)) {
		for (ils in 1:3) {
			xxnames[n] <- paste0(nindivs, ",", g, ",", ils.vals[ils])
			ratios[n,1] <- results[4*n - 3, "ratio"]
			ratios[n,2] <- results[4*n - 2, "ratio"]
			ratios[n,3] <- results[4*n - 1, "ratio"]
			ratios[n,4] <- results[4*n, "ratio"]
			migs[n,1] <- results[4*n - 3, "migs.pbp"]
			migs[n,2] <- results[4*n - 2, "migs.pbp"]
			migs[n,3] <- results[4*n - 1, "migs.pbp"]
			migs[n,4] <- results[4*n, "migs.pbp"]
			n <- n+1
		}
	}
}



plot.results <- function(xx, yy) {
	par(mfcol=c(2,1))
	par(bg="white")
	
	matplot(xx,log(ratios), pch=c(4, 1, 4, 1), cex=c(0.9,0.9,1.8,1.8), col="black", type="p", xaxt="n", xlab="", ylab="log(ratio)", ylim=c(-0.2,3.2))
	text(c(2,5,8,11), 2.95, labels=c("i=3, g=3", "i=3, g=9", "i=9, g=3", "i=9, g=9"))
	axis(side=1, labels=rep(c("S", "M", "L"),4), at=1:12)
	abline(v=c(3.5,6.5,9.5))
	
	
	matplot(xx, migs, pch=c(4, 1, 4, 1), cex=c(0.9,0.9,1.8,1.8),  col="black", type="p", xaxt="n", xlab="", ylab="migrations", ylim=c(0,1.08))
	text(c(2,5,8,11), 1.02, labels=c("i=3, g=3", "i=3, g=9", "i=9, g=3", "i=9, g=9"))
	axis(side=1, labels=rep(c("S", "M", "L"),4), at=1:12)
	abline(v=c(3.5,6.5,9.5))
}

plot.results(xx,yy)

pdf(file=paste0(xml.fragments.dir, "/prior-results.pdf"), width=5, height=5)
par(mar=c(2,4,1,1)+.1)
plot.results(xx,yy)
dev.off()






#      nspp nindivs nloci   ils model nseqs sHeight  ratio    ESS mean.migs migs.pbp
# [1,]    6       3     3 5e-04     1    54  0.0154  1.064  186.3    0.1260 0.007412
# [2,]    6       3     3 5e-04    -1    54  0.0139  0.961  135.9    0.1909 0.011230
# [3,]    6       3     3 5e-04     1    54  0.0140  0.969  225.3    0.5642 0.033188
# [4,]    6       3     3 5e-04    -1    54  0.0196  1.351*  181.7    6.1331 0.360773 *
# [5,]    6       3     3 5e-03     1    54  0.0149  1.025  188.5    0.0647 0.003804
# [6,]    6       3     3 5e-03    -1    54  0.0144  0.996  135.4    0.0806 0.004741
# [7,]    6       3     3 5e-03     1    54  0.0155  1.068  200.9    0.2672 0.015720
# [8,]    6       3     3 5e-03    -1    54  0.0179  1.234  316.9    1.4142 0.083186
# [9,]    6       3     3 5e-02     1    54  0.0149  1.028  315.6    0.0164 0.000965
# [10,]    6       3     3 5e-02    -1    54  0.0150  1.036  294.9    0.0193 0.001133
# [11,]    6       3     3 5e-02     1    54  0.0140  0.968  344.3    0.0706 0.004154
# [12,]    6       3     3 5e-02    -1    54  0.0145  1.000  284.5    0.0877 0.005161

# [13,]    6       3     9 5e-04     1   162  0.0134  0.924  107.2    0.1277 0.007512
# [14,]    6       3     9 5e-04    -1   162  0.0221  1.526*   56.9    3.6791 0.216417 *-
# [15,]    6       3     9 5e-04     1   162  0.0160  1.104  103.2    0.5983 0.035192
# [16,]    6       3     9 5e-04    -1   162  0.0335  2.307**  269.8   13.2444 0.779081 *
# [17,]    6       3     9 5e-03     1   162  0.0156  1.074   59.4    0.0678 0.003986
# [18,]    6       3     9 5e-03    -1   162  0.0169  1.169   63.1    0.1840 0.010821
# [19,]    6       3     9 5e-03     1   162  0.0161  1.108   81.7    0.3198 0.018812
# [20,]    6       3     9 5e-03    -1   162  0.0467  3.219**  901.3   12.5423 0.737785 *
# [21,]    6       3     9 5e-02     1   162  0.0154  1.064   67.1    0.0196 0.001154
# [22,]    6       3     9 5e-02    -1   162  0.0164  1.132   57.2    0.0242 0.001424
# [23,]    6       3     9 5e-02     1   162  0.0161  1.112   86.6    0.0784 0.004614
# [24,]    6       3     9 5e-02    -1   162  0.0696  4.801** 1385.9   10.3082 0.606366 *

# [25,]    6       9     3 5e-04     1   162  0.0145  1.001   61.3    0.3503 0.006609
# [26,]    6       9     3 5e-04    -1   162  0.0179  1.237   65.4    6.8541 0.129323 *-
# [27,]    6       9     3 5e-04     1   162  0.0173  1.194   59.7    1.7885 0.033744
# [28,]    6       9     3 5e-04    -1   162  0.0243  1.678*  868.1   40.2014 0.758517 *
# [29,]    6       9     3 5e-03     1   162  0.0148  1.019  117.0    0.2544 0.004799
# [30,]    6       9     3 5e-03    -1   162  0.0160  1.101  182.2    1.1624 0.021932
# [31,]    6       9     3 5e-03     1   162  0.0170  1.172  203.8    1.1443 0.021591
# [32,]    6       9     3 5e-03    -1   162  0.0344  2.372** 1780.7   39.0774 0.737309 *
# [33,]    6       9     3 5e-02     1   162  0.0146  1.007   69.2    0.0999 0.001885
# [34,]    6       9     3 5e-02    -1   162  0.0163  1.124  101.6    0.1500 0.002830
# [35,]    6       9     3 5e-02     1   162  0.0165  1.139  119.9    0.4832 0.009117
# [36,]    6       9     3 5e-02    -1   162  0.0551  3.800** 1825.3   35.1029 0.662319 *

# [37,]    6       9     9 5e-04     1   486  0.0159  1.095   29.5    0.3915 0.007387
# [38,]    6       9     9 5e-04    -1   486  0.0387  2.672**  126.9   36.0800 0.680754 *
# [39,]    6       9     9 5e-04     1   486  0.1137  7.840**   73.2   42.6424 0.804574 * 1
# [40,]    6       9     9 5e-04    -1   486  0.0316  2.178**  671.3   48.7293 0.919420 *
# [41,]    6       9     9 5e-03     1   486  0.0166  1.148   45.7    0.3019 0.005696
# [42,]    6       9     9 5e-03    -1   486  0.0483  3.333**  563.5   35.5578 0.670901 * 1
# [43,]    6       9     9 5e-03     1   486  0.0189  1.300   54.9    1.8144 0.034234
# [44,]    6       9     9 5e-03    -1   486  0.0453  3.127** 1079.2   48.5485 0.916009 *
# [45,]    6       9     9 5e-02     1   486  0.0173  1.193   33.1    0.1331 0.002511
# [46,]    6       9     9 5e-02    -1   486  0.0758  5.229** 1032.0   33.2686 0.627709 *
# [47,]    6       9     9 5e-02     1   486  0.2038 14.054**  266.2   40.3785 0.761859 * 1
# [48,]    6       9     9 5e-02    -1   486  0.0717  4.942** 1402.0   47.7254 0.900479 * 






